define(['config', 'jquery', 'ajax_queue', 'storage', 'filterable-list', 'mixins', 'utils', 'throttle-debounce'], function(ECMWF, $, Ajaxqueue, localStorage, List, Mixins){

	/*
	 * Facets are formatted as below:
	 *
	 * product: {
	 *     tags: {
	 *         facet_name1: value1,
	 *         facet_name2: [value2, value3]
	 *     }
	 * }
	 *
	 */

	var settings = {
		cache_expire_mins: ECMWF.options.thumbnail_cache || 10
	}

	var mixins = {
		validate: function(tests) {
			for(var t in tests) if(!tests[t][1]()){
				var message = tests[t][0];
				console.warn('validate: "' + message + '" failed.');
				this.error_msg = "test '" + message + "' failed";
				return false;
			}
			return true;
		}
	}

	function Collection(){
	}
	Collection.prototype = $.extend(new List(), mixins, {bind: Mixins.bind}, {
		set_package: function(pkg){
			this._products = null;
			this._facets = null;
			$.extend(this.query, {package: pkg})
		},

		set_list: function(){
			var self = this;
			this.products().then(function(items){
				self.on_load(items)
			})
		},

		product_by_name: function(name) {
			for(var i in this._products){
				if(this._products[i].data.name === name) return this._products[i];
			}
		},

		/*
		 *  Get html text content associated with the given product or category.
		 */
		content: function(product_name, done) {
			Ajaxqueue({
				url: '/plots/get-product-text/',
				json: {product: {name: product_name}},
				success: done
			});
		},

		/*
		 *  Generate a list of facets from the product list.
		 *  The method is synchronous and relies only on product data, not package data.
		 */
		facets: function(){
			var self = this;
			if(!this._facets){
				this._facets = {};

				$.each(this._products, function(i, product){
					if(product.data.tags){
						$.each(product.data.tags, function(i, v){

							self._facets[i] || (self._facets[i] = {});

							if(Array.isArray(v))
								$.each(v, function(j, v){ self._facets[i][v] = true; });
							else
								self._facets[i][v] = true;
						});
					}
				});
				$.each(self._facets, function(i, v){
					self._facets[i] = $.map(v, function(_, i){ return i; }).sort();
				});
			}

			return this._facets;
		},

		/*
		 * Now that products can be in more than one package, the pkg needs to be specified
		 */
		sorted_facets: function(pkg, blacklist, done) {
			var self = this;
			if(arguments.length === 2){
				done = blacklist;
				blacklist = [];
			}

			this._products.length && this.packages.then(function() {
				self.facets();

				var sorted = pkg.facets || [];

				var keys = sorted.length
					? $.map(self._facets, function(i, key){ if(!~$.inArray(key, blacklist)) return {name: key, values: i, order: $.inArray(key, sorted)}; })
					: $.map(self._facets, function(i, key){ if(!~$.inArray(key, blacklist)) return {name: key, values: i, order: 0}; });

				done(keys.sort(function(a, b) {
					return a.order - b.order;
				}));
			});
		},

		cancel_thumbnails: function() {
			//Ajaxqueue.cancelByTag(new RegExp(self.eventNamespace + '$')); - does not work when batching requests
			this._cancel_warning_done || console.log('products.cancel_thumbnails TODO');
			this._cancel_warning_done = true;
		}
	})

	return Collection
})

