define(['jquery', 'bus'], function($, Bus) {

	var max = 50,
		undoes = [],
		redoes = [],
		bus = Bus();

	function notify() {
		bus.send('undo-list-changed', { undo: undoes.length, redo: redoes.length });
	}

	var Undo = {
		store: function(owner, undo, redo) {
			undoes.push({ owner: owner, undo: undo, redo: redo });
			redoes = [];

			while (undoes.length > max) {
				undoes.shift();
			}

			notify();

			return undoes.length;
		},
		length: function() {
			return undoes.length;
		}
	};

	$.undoClearLast = function() {
		undoes.pop();
	};

	$.undoLast = function() {
		if (undoes.length) {
			var x = undoes.pop(),
			f = x.undo(); // The function can also return a "redo" action

			x.redo = f || x.redo;
			if (x.redo) {
				redoes.push(x);
			}
			else {
				redoes = [];
			}
		}
		notify();
	};

	$.redoLast = function() {
		if (redoes.length) {
			var x = redoes.pop();
			x.redo();
			undoes.push(x);
		}
		notify();
	};

	$.undoRemoveAll = function(owner) {
		// For now...
		undoes = [];
		redoes = [];
		notify();
	};

	bus.listen('undo-key-press', $.undoLast);
	bus.listen('redo-key-press', $.redoLast);

	return Undo;
});
