define(['config', 'jquery', 'bus', 'ajax_queue', './reporter', 'settings', 'chart/plugins/resize', 'unit', 'thumbnail-list', 'messages', 'chart', 'chart/plugins/marker', 'chart/plugins/save', 'toolbar', 'toolbar/extended.css', 'product_manager', 'tooltip/tipsy', 'meteograms', 'focusable', 'ajax-indicator', 'chart/latlon', './forecaster.css', './application.css', 'theme-common/charts.css'], function(ECMWF, $, Bus, Ajaxqueue, Reporter, Settings, resize) {

var old_template;

$.when(Settings.schema, Settings.data).then(function(){
	if (!ECMWF.request.kiosk) {
		$.fn.keyboard = function() {
			return this;
		}
	}

	window.Workspaces && Workspaces.init();

	$(function(){
		ECMWF.wheelspeed = ECMWF.browser.mozilla ? 10 : 1;
		old_template = !$('#app-menu').length;

		$.fn.workspaceMenu && $('#menu-bar').workspaceMenu({menu: new ECMWF.MenuBar('#menu-bar', 'menu-panes'), position: 5});

      load_chart(function(_chart){
			chart = _chart;

			(chart && chart.create()) || ECMWF.alert({
				title: 'Chart not found',
				content: (
					ECMWF.request.widget
					? 'Cannot find widget "<b>' + $('<span>').text(ECMWF.request.widget).text() + '</b>"'
					: 'Chart not found: ' + $('#chart').chart('option', 'product')
				)
			});
		});

		/*
		 *  In order to speed up the initialisation of the chart, both
		 *  the loading and execution of secondary items is delayed
		 */
		require(['./stage2'])

		$(document).productManager()

		Reporter();

		Bus()
			.listen('multigrams-list-changed', function (_, msg) {
				Settings.set('meteograms-list', msg);
				$.logMessage('Meteogram List updated');
			})
			.listen('menu-help-help-select', function (_, msg) {
				window.open(ECMWF.options.settings.eccharts_help);
			})
			.listen('menu-help-preferences-select', function (_, msg) {
				require('preferences') && $(window).preferences();
			})
			.listen('menu-go-dashboard-select', function (_, msg) {
				location.href = ECMWF.options.site.dashboard;
			})
			.listen('chart-content-update', function (_, msg) {
				msg.changed.get('all') && chart.save(msg);
			})
			.listen('dialogue-state-changed', function (_, msg) {
				var pref = Settings.get('dialogue-states', {});
				pref[msg.name] = msg;
				Settings.set('dialogue-states', pref);
			})
			.listen('latlon-location-select', function (event, msg) {
				if(msg.from !== 'init'){
					Settings.set('last-probe-location', msg);
				}
				$.logMessage($.formatGeoLocation(msg) + ' ' + (msg.title || ''));
			})
			.listen('presentation', function () {
				$('.no-presentation').toggle();
				$('body').triggerHandler('layout');
			})
			.listen('workspace-select', function (_, workspace) {
				$.logMessage('Switched to workspace: ' + workspace.title);
				$('#chart').chart('setState', Settings.get('last-product'));
			});

		var loc = Settings.get('last-probe-location', null);
		if (loc) {
			setTimeout(function () {
				loc.from = 'init';
				Bus().send('latlon-location-select', loc);
			},
			500);
		}

		if(false && old_template){
			$('body').container({
				layout: 'border',
				north: $.extend($('.header-bar, #content-heading, #menu-bar, #page-wrapper > header'), {autoHeight: true}), // TODO move template specifics to config
				center: $(ECMWF.options.settings.content || '#content'),
				south: $('#status-bar')
			});
		}else{
			// use a custom layout for now until we know what is needed
			$('body').container({
				layout: function() {
					if(__BOOTSTRAP__)
						return {
							layout: function(size) {
								var elem = $('body')
								size = {width: elem.width(), height: elem.height() - 70};
								elem.children().content().each(function(){}).triggerHandler('layout', size);
								$('#gadget').triggerHandler('layout', size);
							}
						}
					else
						// new style for existing www page
						return {
							layout: function(size) {
								var bheight = 70;

								if(!$('header').is(':visible')) bheight -= 47
								if(!$('#status-bar-outer').is(':visible')) bheight -= 32

								$('#gadget').height($('body').height() - bheight)
								$('#gadget-content').height($('body').height() - bheight - 33)

								var elem = $('body')
								size = {width: elem.width(), height: elem.height() - bheight - 33};
								elem.children().content().each(function(){}).triggerHandler('layout', size);
								$('#chart').triggerHandler('layout', size);
								$('#time-navigator .time-nav').triggerHandler('layout', size);
							}
						}
				},
				north: $.extend($('.header-bar, #content-heading, #menu-bar, #page-wrapper > header'), {autoHeight: true}), // TODO move template specifics to config
				center: $(ECMWF.options.settings.content || '#content'),
				south: $('#status-bar')
			});
		}

		if(false && old_template){
		$('#gadget').container({
			layout: 'border',
			north: $('#gadget-head'),
			center: $('#gadget-content'),
			south: $('#time-navigator')
		});
		}

		$.fn.focusable();

		$('#ajax-loader').ajaxIndicator();

		function open_dialogue(kind){
			self.dialogues = self.dialogues || {}
			require(['thumbnail-dialogue/2'], function(Dialogue){
				self.dialogues[kind] = self.dialogues[kind] ? self.dialogues[kind].open() : new Dialogue({kind: kind})
			});
		}

		var chart;
		function load_chart(done){

			if (ECMWF.request.widget) {

				Ajaxqueue({
					url: ECMWF.options.ajax.backend.get_widget,
					json: {name: ECMWF.request.widget},
					success: function (result) {
						done(new dashboardChartDelegate(result));
					},
					error: function (result) {
					}
				});
			}
			else if (ECMWF.request.product) {
				done(new presetChartDelegate());
			} else {
				done(new chartDelegate());
			}
		}

		function chartDelegate() {

			this.options = {
				projection: (ECMWF.request.projection || 'europe'),
				use_outline: true,
				plugins: [resize]
			};

			this.save = function (msg) {
				Settings.set('last-product', msg.product.get_save_state());
			}

			this.create = function () {
				this.options.product = Settings.get('last-product', 'wind');
				return $('#chart').chart($.extend(this.options, {
					title: $('#chart-title')
				}));
			}
		}

		function presetChartDelegate() {

			this.options = {
				product: ECMWF.request.product,
				projection: (ECMWF.request.projection || 'europe'),
				plugins: [resize]
			};

			this.save = $.noop; // not savable

			this.create = function () {
				console.log('presetChartDelegate: product=', this.options.product);
				return $('#chart').chart($.extend(this.options, {
					title: $('#chart-title')
				}));
			}
		}

		function dashboardChartDelegate(widget) {

			ECMWF.request.widget = widget;

			this._title = widget.title;
			console.log('dashboardChartDelegate: widget.data:');
			console.log(widget.data);

			this.options = {
				product: widget.data.product,
				projection: widget.data.projection,
				plugins: [resize]
			};

			this.save = function (msg) {
				if (Settings.get('auto-save-widgets', false)) {
					widget.data.product = msg.product.save();
					Ajaxqueue({
						priority: 'very-high',
						url: ECMWF.options.ajax.backend.save_widget,
						json: widget,
						logErrors: 'Save product',
						success: function (result) {
							console.log('done. ', result);
						}
					});
				}
			}

			this.create = function () {
				console.log('dashboardChartDelegate.create: product=', this.options.product);
				var c = $('#chart').chart(this.options);
				$('#gadget').triggerHandler('layout');
				return c;
			}
		}
	})
})
});
