/*
 *  ECWMF Projections
 *  -----------------
 *
 *  A generic collection object for chart projections.
 *
 *  Please note that this conforms to the Product Collection Interface (see documentation). Do not modify the API.
 *
 */
define(['config', 'jquery', 'ajax_queue', 'storage', 'bus', 'mixins', 'collection', 'collection/item', 'utils', 'throttle-debounce'], function(ECMWF, $, Ajaxqueue, localStorage, Bus, Mixins, Collection, ChartListItem){

	/*
	 * Facets are formatted as below:
	 *
	 * product: {
	 *     tags: {
	 *         facet_name1: value1,
	 *         facet_name2: [value2, value3]
	 *     }
	 * }
	 *
	 */

	ECMWF.options.timeout && $.extend(Ajaxqueue.options, {
		timeout: ECMWF.options.timeout * 1000
	});

	function Projections(){

		this.product_type = 'eccharts' // used by vgrid
		this.query = {}
		this.thumbnail_priority = 'low'

		this.list_init()

		this.products = function() {
			var self = this;

			// TODO change so that it does not return a new promise each time
			return $.Deferred(function(df) {

				function have_products(result) {
					if(window._ECMWF && _ECMWF.all_products){
						$.each(_ECMWF.all_products, function(i, v){
							var p = $.findWhere(result, {name: v.name})
							if(p){
								p.thumbnail = v.thumbnail
							}
						})
					}

					__DEV__ && Bus().send('event-start', ['start verification']);
					self._products = $.map(result/*.records*/, function(item){
						if(self.validate([
								['not empty', function(){ return item; }],
								['has name', function(){ return item.name; }],
								['has title', function(){ return item.title; }],
							])){
							var product = new ChartListItem(item);
							if((!__DEV__) || (product.cache.score.apply(product) > -1)) return product;
						}
					});
					if(self._products.length){
						var rejected = result.query_count - self._products.length
						console.log('n_products=' + self._products.length + (rejected ? ' (' + (rejected) + ' rejected)' : ''))
					}else{
						console.log('no products found for:', query || self.query)
					}

					//self.packages.then(function() {
						df.resolve(self._products)
					//});
				}

				var user_only = false

				Ajaxqueue({
					url: '/backend/catalogue-items/',
					json: {
						type: this.kind,
						user_items: user_only,
						standard_items: !user_only,
						allow_experimental: ECMWF.request.allow_experimental,
						access: ECMWF.request.access,
						keywords: true,
						descriptions: true
					},
					cache: 30,
					tag: self.name,
					complete: have_products
				})
			}).promise()
		}

		this.hierarchy = {
			products: this,
			tree: {},

			update: function(categories) {
				var self = this;
				/*
				this.tree = {};
				$.each(this.products.products, function(i, p){
					for(var k in p.data.path){
						var path = p.data.path[k].split('/');
						var cursor = self.tree;
						for(var j in path){
							if(path[j]){
								cursor = cursor[path[j]] = cursor[path[j]] || {};
							}
						}
					}
				});
				*/
				this.tree = categories;

				// fill in missing categories TODO move to server.
				for(var i in this.products._packages){
					var pkg = this.products._packages[i];
					if(!$.findWhere(this.tree, {name: pkg.name})){
						this.tree.push({name: pkg.name, title: pkg.title});
					}
				}
			},

			children: function(path) {
				var node = this._child(path, 0, this.tree);
				return node ? node.children : [];
			},

			lookup: function(path) {
				if(!path || path === 'all') return {title: 'ECMWF charts'}; // special case because the tree does not currently have a root item

				$.isEmptyObject(this.tree) && console.warn('hierarchy.lookup: no package list');
				if(!Array.isArray(path)) path = path.replace(/\/#.*/, '').split('/');
				return this._child(path, 0, this.tree) || $.findWhere(products._packages, {name: ECMWF.options.default_package})
			},

			_child: function(path, p, tree){
				for(var i in tree){
					var node = tree[i];
					if(node.name === path[p]){
						if(p === path.length - 1){
							return node;
						}else{
							p++;
							return this._child(path, p, node.children);
						}
					}
				}
			},

			path_is_valid: function(path) {
				return true;
			}
		}

		this._store = $.debounce(2000, function() {
			localStorage.products = JSON.stringify(this.__cache);
		})

		this.store = localStorage ? function(name, item) {
			this.__cache[name] = item;
			this._store();
		} : $.noop;
	}

	Projections.prototype = $.extend(new Collection(), {
		kind: 'projection',

		/* not used ?
		item_by_name: function(name) {
			for(var i in this.products){
				if(this.products[i].data.name === name) return this.products[i];
			}
		}
		*/
	})

	var products
	return products = new Projections();
});
