define(['config', 'jquery', 'settings', 'shortcuts', 'time-navigator'], function(ECMWF, $, Settings) {

	var fns = [
		function(){
			ECMWF.browser.supports_svg_img(function(supported) {
				if(!supported) {
					console.warn('svg-image NOT supported');
					// see widgets/chart/plugins/svg_image.js for better implementation
					if(Settings.get_pref('image_format') === 'svg'){
						ECMWF.alert({
							title:   'SVG Not Supported',
							content: '<b>This browser does not support SVG images.</b><p style="margin-top:1em;">Please go to Help / Preferences and change Image Format to PNG.</p>'
						});
					}
				}
			});
		},
		function(){
			$('#products').thumbnailList({
				kind: 'product',
				visible_items: 6,
				show_add_button: function () {
					if(false)
						$('#products').thumbnailDialogue(product_thumbnail_dialogue_options);
					else
						open_dialogue('product')
				}
			});
			$('#projections').thumbnailList({
				kind: 'projection',
				visible_items: 6,
				show_add_button: function () {
					if(false)
						$('#projections').thumbnailDialogue(projection_thumbnail_dialogue_options);
					else
						open_dialogue('projection')
				}
			});

			$('#views, #save, #test').menuList();
			$('#go').menuList().find('li:first').removeClass('enabled').addClass('disabled');
			$('#messages').messages();
			//$("#timers").timersInfo();
			//$("#notifications").notifications({ counter: $("#notifications-counter")});
			require(['data-availability'], function() {
				$('#availability').dataAvailability({
					counter: $('#availability-counter')
				})
			})
		},
		function(){
			// Create all menus
			require(['menu_pane'], function() {
				var menus = ['products', 'projections', 'views', 'save', 'availability', 'test', 'go'];
				for (var i in menus) {
					$('#' + menus[i] + '-menu').addClass('fff').button({icons: {secondary: 'ui-icon-triangle-1-s'}});

					$('#' + menus[i] + '-pane').menuPane({
						group: '#menu-panes > div',
						owner: $('#' + menus[i] + '-menu')
					});

					// When a thumbnail is selected, close the menu
					(function (menu) {
						$('#' + menu).on('select', function () {
							$('#' + menu + "-pane").trigger('close');
							return false;
						});
					})(menus[i]);
				}

				var fs = {
					button: $('#menu-bar').li('.menu-item').text('Fullscreen').button({iconPosition: 'end'}).click(function(){
						$('#status-bar-outer, header').toggle();
						fs.handlers[fs.i++ % 2]();
						$('body').triggerHandler('layout');
					}),
					handlers: [
						function() {
							$('#main').addClass('fullsize');
							fs.button.button('option', 'icons', {primary: 'ui-icon-arrow-2-ne-sw'});
						},
						function() {
							console.log('2');
							$('#main').removeClass('fullsize');
							fs.button.button('option', 'icons', {primary: 'ui-icon-arrow-4-diag'});
						}
					],
					i: 0
				}
			})
		},
		/* fs button is currently being tested on the menu instead of having a button
		function(){
			var fs = {
				button: $('<div class="fullscreen-button">').button({icons: {primary: 'ui-icon-arrow-4-diag'}, text: false, label: 'Full size'}).prependTo('#gadget-head')
					.click(function(){
						$('#status-bar-outer, header').toggle();
						fs.handlers[fs.i++ % 2]();
						$('body').triggerHandler('layout');
					}),
				handlers: [
					function() {
						$('#main').addClass('fullsize');
						//fs.button('option', 'icons', {primary: 'ui-icon-arrowthick-2-ne-sw'});
						fs.button.button('option', 'icons', {primary: 'ui-icon-arrow-2-ne-sw'});
					},
					function() {
						console.log('2');
						$('#main').removeClass('fullsize');
						fs.button.button('option', 'icons', {primary: 'ui-icon-arrow-4-diag'});
					}
				],
				i: 0
			}
		},
		*/
		function(){
			import('./tools').then(function() {
				$('#toolbar').toolbar({
					tools: [
						['clear', 'full-map'],
						['undo', 'redo'],
						['zoom-in', 'zoom-out'],
						['drag-pan', 'drag-zoom', 'select', 'vertical'],
						['goto-marker'],
						['print']],
					tooltips: true
				})
			})
		},
		function(){
			require(['tool-buttons/layers'], function() {
				$('#toolbar-right').toolbar({
					tools: [
						['layers']
					],
					tooltips: true
				})
			})
		},
		function(){
			require(['menu_pane', 'layer-list'], function() {
				$('#layer-list').layerList();
				$('#layer-list-pane').menuPane({
					noWidth: true,
					align: 'right',
					owner: $('.button.layers')
				})
			})
		},
		function(){
			$('#time-navigator').timeNavigator().hide();

			setTimeout(function(){
				$('#time-navigator').show().animate({opacity: 1}, function(){
					$('#time-navigator').find('> .time-nav').triggerHandler('layout');
				});
			}, 500);
		},
		function(){
			require(['animation-controls'], function() {
				var states = Settings.get('dialogue-states', {});
				$('body').animationControls({state: states['controls']});
			})
		},
		function(){
			// on-demand dialogue instantiation
			require(['view-dialogue', 'cities', 'legends', './time_series', './big_title'], function() {
				$.each({
						probe:         { title: 'Probe'},
						meteograms:    { title: 'Meteograms', options: Settings.get('meteograms-list', {products: ['10d_epsgram_2t'] })},
						legends:       { options: {show_zoom_buttons: true}},
						cities:        { title: 'City Finder'},
						info:          { widget: 'bigTitle'},
						'time-series': { title: 'Time Series', widget: 'chartTimeSeries'}
					},
					$.viewDialogue
				)
			})
		}
	]
	var i = -1;
	(function next(){
		var fn = fns[++i];
		fn && setTimeout(function(){
			fn()
			next()
		}, 10)
	})()

	$(document).shortcuts({
		shortcuts: [{
			key: $.ui.keyCode.UP,
			signal: 'next-base-time'
		},
		{
			key: $.ui.keyCode.DOWN,
			signal: 'prev-base-time'
		},
		{
			key: $.ui.keyCode.LEFT,
			signal: 'prev-step'
		},
		{
			key: $.ui.keyCode.RIGHT,
			signal: 'next-step'
		},
		{
			key: $.ui.keyCode.HOME,
			signal: 'first-step'
		},
		{
			key: $.ui.keyCode.END,
			signal: 'last-step'
		},
		{
			key: $.ui.keyCode.ESCAPE,
			signal: 'cancel'
		},
		{
			key: 52, // '4'
			callback: function () {
				open_dialogue('projection')
			}
		},
		{
			key: 53, // '5'
			callback: function () {
				open_dialogue('product')
			}
		},
		{
			key: 54, // '6'
			signal: 'open-layer-thumbnail-dialogue'
		},
		{
			key: $.ui.keyCode.NUMPAD_ADD,
			signal: 'tool-zoom-in'
		},
		{
			key: $.ui.keyCode.NUMPAD_SUBTRACT,
			signal: 'tool-zoom-out'
		},
		{
			key: $.ui.keyCode.COMMA,
			signal: 'animation-speed-down'
		},
		{
			key: $.ui.keyCode.PERIOD,
			signal: 'animation-speed-up'
		},
		{
			key: 51, // '#' key
			callback: function() { // '3'
				require(['chart/plugins/timers'], function(plugin) {
					var chart = $.fn.chart && $('.chart').chart('instance') || ($.fn.basicChart && $('.basic').basicChart('instance'));
					if(chart && chart.add_plugin('Timers', plugin)){
						require(['notify'], function() {
							$.notify({id: 1, kind: 'ok', story: 'ok', message: 'Calltree timers are now enabled<br/>Subsequent requests will have timer information available.'});
						});
					}else{
						Bus().send('open-calltree');
					}
				});
			}
		},
		{
			key: 61, // '=' key
			signal: 'show-timers'
		},
		{
			key: 187, // '=' key (webkit)
			signal: 'show-timers'
		},
		{
			key: 70, // 'f'
			signal: 'fullscreen'
		},
		{
			key: 80, // 'p'
			signal: 'presentation'
		},
		{
			key: 85, // 'u'
			signal: 'undo-key-press'
		},
		{
			key: 82, // 'r'
			signal: 'redo-key-press'
		}]
	});
})
