define(['jquery', 'grid/filter'], function($, Filter){

	return new Filter({
		name: 'facets',

		fn: function(item){
			if($.isEmptyObject(this._active)) return true;
			var facets = item.data.tags;
			if(!facets) return false;

			for(var facet in this._active){
				if(!facets[facet]) return false;
				if(!$.map(this._active[facet], function(val){ if(!!~$.inArray(val, facets[facet])) return val; }).length) return false;
			}
			return true;
		},

		set: function(prop, active) {
			var f = prop[0];
			var fval = prop[1];
			console.log('filters.facets.set: ', f, fval, active);

			if(active){
				this._active[f] = this._active[f] || [];
				!!~$.inArray(fval, this._active[f]) || this._active[f].push(fval);
			}else{
				this._active[f] = $.map(this._active[f], function(v){ if(v !== fval) return v; });
				if(!this._active[f].length) delete this._active[f];
			}
			this.events.send('changed', [prop, active]);
		},

		deactivate: function() {
			console.log('filters.facets.deactivate');
			this._active = {};
			this.events.send('changed', []);
		},

		status: function() {
			return $.map(this._active, function(v, i) {
				return i + ': ' + v.join(', ');
			}).join(' / ');
		},

		// example url query: ?facets=Product group,Basic EFI;Component,Atmosphere,Geography
		to_url: function() {
			return {
				facets: $.map(this._active, function(v, i) {
					return i + ',' + v.join(',');
				}).join(';')
			}
		},

		_active: {}
	});
});
