/*
 *   This is intended to be a next-gen replacement for thumbnail-dialogue/index.js
 *
 *   TODO 
 *    - need to support both 'add to list' and 'view on map' (currently does only add-to-list)
 *        - show links on hover?
 *    - scrollbar size not updated after add/remove user_products filter
 *    - group headings not being shown
 *    - when opening dialogue a 2nd time, need to re-run the filter
 */
define(['jquery', 'bus', 'utils', 'settings', 'mixins', 'observable', 'vgrid', 'forecaster/products', 'forecaster/projections', 'grid/filter', 'grid/filters/text', 'dialogue2013', 'virtual-dom/h', 'virtual-dom/diff', 'virtual-dom/patch', 'vdom-virtualize', 'render-tree'], function($, Bus, Utils, Settings, mixins, Observable, VGrid, Collections, projections, Filter, text_filter, Dialogue, h, diff, patch, virtualise, RenderTree) {

var setting = 'thumbnail_dialogue_hide_items_you_already_have'

var UserProducts = function(o){
	var self = this
	this.collection = o.collection
	this.products = []

	this.filter = new Filter({
		name: 'user-products',
		fn: function(item){
			// we determine if an ECMWF product is in the user product list by comparing title
			// we cannot use name, because the user products have the user name prepended
			var title = item.title_()
			for(var i in self.products.data){
				if(self.products.data[i].title === title) return false;
			}
			return true;
		},
		deactivate: function() {
			Settings.set(setting, false)
			self.collection.remove_filter(this.name);
		},
		status: function(){ return self.products.data.length + ' user products'; },
		text: null
	})
}

return function(options) {
	var grid, user_products

	var o = $.extend({
		kind: 'product',
		hide_user_items: true
	}, options);

	var dialogue = new Dialogue({
		id: o.kind + '-select',
		title: '<h2>' + Utils.ucfirst(o.kind) + ' select</h2>'
	}).getElements();

	var el = dialogue.modal;//$('#' + o.kind + '-select');
	var height = $(window).height() - 80;
	var width = Math.min(800, $(window).width() - 80)
	el.css({width: width, height: height, marginTop: -height/2, marginLeft: -width/2}).modal('show')
	dialogue.footer.hide()

	var inner_render = function(){ return h('div') }

	var products = (o.kind === 'product') ? Collections.products : projections;
	var first_time = !dialogue.body.find('> *').length;
		// TODO remove first time, but check list is initialised
		if(!first_time) console.error('expected first_time');

	$.Deferred(function(df){
		if(o.hide_user_items){
			user_products = new UserProducts({collection: products})
			require(['user_catalogue'], function(UserCatalogueStore) {
				user_products.products = new UserCatalogueStore()
					.create(o)
					.get_catalogue(df.resolve)
			})
		} else df.resolve();
	}).then(function(){
		self.vdom = new RenderTree({
			main: {
				render: function(){
					return h('.modal-body', [
						h('p', ['Select from these ECMWF ' + Utils.ucfirst(o.kind) + 's to add to your personal list']),
						inner_render(),
						h('.check-options', [
							h('input', {type: 'checkbox', checked: Settings.get(setting) && 'checked', onclick: function(e){
								var checked = e.target.checked
								if(checked)
									products.add_filters([user_products.filter]);
								else
									products.remove_filter(user_products.filter.name);
								Settings.set(setting, checked);
							}}),
							h('label', ['Hide ' + Utils.ucfirst(o.kind) + 's you already have'])
						])
					])
				}
			},
			root: dialogue.body[0],
			tree: virtualise(self.dom_tree_root = el.find('.modal-body')[0])
		})

		if(first_time){
			$('#product-select').on('render', self.vdom.bind(self.vdom.render))

			products.subscribe(self.vdom.bind(self.vdom.render))
			//products.set_collection(o.kind === 'product' ? products : projections); // triggers product.fetch
			products.set_list()

			products.on.items.value = 1, products.on.items.options.state = true; // TODO observable does not have a value?
			products.on.items.subscribe(self, function() {
				products.filter(); // TODO should be done inside the list
			});

			// TODO this duplicates fn in grid.js - should be done in the list, not in the grid
			$.map(products.filters, function(f){
				$.isFunction(f.fn) && f.events && f.events.bind('changed', function() {
					products.filter();
					self.vdom.render();
				});
			});

			self.vdom.on.render.once(function(){
				var v_space = height - el.find('.modal-header').outerHeight() - dialogue.body.outerHeight() - el.find('.check-options').outerHeight();

				// TODO we need to support both 'add to list' and 'view on map'
				var grid = new VGrid({
					renderer: self.vdom,
					products: products,
					width: $('.modal-body').width(),
					height: v_space,
					on_click: function(selection){
						console.log('ThumbnailDialogue2: closing... selection=', selection);
						el.modal('hide');
						Bus().send(o.kind + '-dialogue-selection', {names: [selection.name()]});
					},
					on_dblclick: function(selection){
						console.log('ThumbnailDialogue2: dblclick: selection=', selection);
						el.modal('hide');
						Bus().send('product-thumbnail-click', {name: selection.name()});
					}
				})

				products.have.filtering.then(function() {
					inner_render = grid.bind(grid.render);
				})
			})

			self.vdom.render()
		}
	})

	if(first_time){
		products.add_filters([text_filter]);
		Settings.get(setting) && products.add_filters([user_products.filter]);
	}

	this.open = function() {
		el.modal('show')
	}
}
})
