
define(['jquery', 'tool-button', 'tool-buttons/zoomin', 'tool-buttons/zoomout', 'tool-buttons/drag_pan', 'tool-buttons/drag_zoom', 'tool-buttons/print', 'tool-buttons/zoomreset', 'container', 'www_chart/model/www_product', 'chart'], function($){

$.widget('ui.interactiveChart', {
	options: {
		instance: {},
		products: null,            // product collection object
		preload_animations: true
	},

	_init: function(){
		var self = this, o = self.options;
		var div = self.chart = self.element.find('.www-chart-image-wrapper');

		var title = $('<div>').addClass('chart-title').insertAfter(self.chart);

		var bus = self.element.attr({id: o.instance.data.name}).messageBus();

		self.img = self.element.find('img.www-chart');

		bus.listen('chart-product-change', function _ready(_, product){
			product = product.product;
			if(product.request.name === o.instance.data.product.name){
				// hide the original image
				self.element.find('.prototype').css({opacity: 0}).delayed(1000, function(){ $(this).remove(); });
				bus.forget('chart-product-change', _ready);

				o.instance.data.modifiable.step !== false && self._time_navigator();
			}
		});

		var params = {};
		self.www_product = new WWWProduct({
			name: o.instance.data.product.name,
			axis: params // key/value pairs
			//req['axis']['interactive_projection'] = this.options.projection; // TODO
		});

		div.chart({product: self.www_product, title: title, messageBus: bus, use_outline: false, availability: false});

		self.toolbar = $('<div>').addClass('chart-toolbar')
			.appendTo(self.chart)
			.toolbar({
				tools: [
					['zoom-in', 'zoom-out', 'zoom-reset'],
					['drag-pan', 'drag-zoom'],
					['print'], ['share']
				],
				messageBus: bus,
				tooltips: true
			});

		/*
		var projections = o.instance.data.product.variables.reduce(function(prev, item){
			return (item.name === 'projection') ? item : prev;
		});
		*/

		self.chart.div().wwwChartParameterBar({
			variables: $.map(o.instance.data.product.variables, function(v, i){ return o.instance.data.modifiable[v.name] !== false ? v : null; }),
			click_handlers: {
				projection: function(event, ui, obj){
					bus.send('projection-thumbnail-click', {name: ui.item.text().toLowerCase().replace(' ', '_')});
				},
				threshold: function(event, ui, obj){
					console.log('TODO threshold menu select: wwwproduct=', self.www_product);
					$.extend(self.www_product.request.axis, {threshold: ui.item.text()});
					console.log('threshold.select: axes=', self.www_product.request.axis);
					self.chart.chart('redraw');
				}
			}
		});

		// temporary availability warning
		var got_availability;
		bus.listen_once('chart-product-availability', function(){
			got_availability = true;
		});
		setTimeout(function _check_availability(){
			//require(['inline_message'], function(){
			//	got_availability || self.element.div().insertBefore(self.element).inlineMessage({message: 'Availability information was not sent. Product information in unexpected format. See issue NWEB-88', mode: 'warning'});
			//});
		}, 5000);
	},

	_time_navigator: function(){
		var self = this;
		require(['time-navigator'], function(){
			$('<div>').insertAfter(self.chart).timeNavigator();
		});
	}
});

});
